@@ -218,12 +218,6 @@ class MethodDispatcher(Dispatcher):
             request.handler = cherrypy.NotFound()
 
 
-class WSGIEnvProxy(object):
-    
-    def __getattr__(self, key):
-        return getattr(cherrypy.request.wsgi_environ, key)
-
-
 class RoutesDispatcher(object):
     """A Routes based dispatcher for CherryPy."""
     
@@ -241,10 +235,6 @@ class RoutesDispatcher(object):
         self.mapper = routes.Mapper()
         self.mapper.controller_scan = self.controllers.keys
         
-        # Since Routes' mapper.environ is not threadsafe,
-        # we must use a proxy which does JIT lookup.
-        self.mapper.environ = WSGIEnvProxy()
-    
     def connect(self, name, route, controller, **kwargs):
         self.controllers[name] = controller
         self.mapper.connect(name, route, controller=name, **kwargs)
@@ -268,11 +258,14 @@ class RoutesDispatcher(object):
         
         config = routes.request_config()
         config.mapper = self.mapper
+        if hasattr(cherrypy.request, 'wsgi_environ'):
+            config.environ = cherrypy.request.wsgi_environ
         config.host = request.headers.get('Host', None)
         config.protocol = request.scheme
         config.redirect = self.redirect
         
         result = self.mapper.match(path_info)
+        
         config.mapper_dict = result
         params = {}
         if result:
@@ -320,7 +313,7 @@ class RoutesDispatcher(object):
             
             action = result.get('action', None)
             if action is not None:
-                handler = getattr(controller, action)
+                handler = getattr(controller, action, None)
                 # Get config from the handler 
                 if hasattr(handler, "_cp_config"): 
                     merge(handler._cp_config)
@@ -346,20 +339,37 @@ def XMLRPCDispatcher(next_dispatcher=Dispatcher()):
 def VirtualHost(next_dispatcher=Dispatcher(), use_x_forwarded_host=True, **domains):
     """Select a different handler based on the Host header.
     
-    Useful when running multiple sites within one CP server.
+    This can be useful when running multiple sites within one CP server.
+    It allows several domains to point to different parts of a single
+    website structure. For example:
     
-    From http://groups.google.com/group/cherrypy-users/browse_thread/thread/f393540fe278e54d:
+        http://www.domain.example  ->  root
+        http://www.domain2.example  ->  root/domain2/
+        http://www.domain2.example:443  ->  root/secure
     
-    For various reasons I need several domains to point to different parts of a
-    single website structure as well as to their own "homepage"   EG
+    can be accomplished via the following config:
     
-    http://www.mydom1.com  ->  root
-    http://www.mydom2.com  ->  root/mydom2/
-    http://www.mydom3.com  ->  root/mydom3/
-    http://www.mydom4.com  ->  under construction page
+        [/]
+        request.dispatch = cherrypy.dispatch.VirtualHost(
+            **{'www.domain2.example': '/domain2',
+               'www.domain2.example:443': '/secure',
+              })
     
-    but also to have  http://www.mydom1.com/mydom2/  etc to be valid pages in
-    their own right.
+    next_dispatcher: the next dispatcher object in the dispatch chain.
+        The VirtualHost dispatcher adds a prefix to the URL and calls
+        another dispatcher. Defaults to cherrypy.dispatch.Dispatcher().
+    
+    use_x_forwarded_host: if True (the default), any "X-Forwarded-Host"
+        request header will be used instead of the "Host" header. This
+        is commonly added by HTTP servers (such as Apache) when proxying.
+    
+    **domains: a dict of {host header value: virtual prefix} pairs.
+        The incoming "Host" request header is looked up in this dict,
+        and, if a match is found, the corresponding "virtual prefix"
+        value will be prepended to the URL path before calling the
+        next dispatcher. Note that you often need separate entries
+        for "example.com" and "www.example.com". In addition, "Host"
+        headers may contain the port number.
     """
     from cherrypy.lib import http
     def vhost_dispatch(path_info):
