@@ -80,7 +80,7 @@ class Root:
 cherrypy.config.update({
     'log.error.file': '',
     'environment': 'production',
-    'server.socket_host': 'localhost',
+    'server.socket_host': '127.0.0.1',
     'server.socket_port': 8080,
     'server.max_request_header_size': 0,
     'server.max_request_body_size': 0,
@@ -135,7 +135,7 @@ This is ApacheBench, Version 2.0.40-dev <$Revision: 1.121.2.1 $> apache-2.0
 Copyright (c) 1996 Adam Twiss, Zeus Technology Ltd, http://www.zeustech.net/
 Copyright (c) 1998-2002 The Apache Software Foundation, http://www.apache.org/
 
-Benchmarking localhost (be patient)
+Benchmarking 127.0.0.1 (be patient)
 Completed 100 requests
 Completed 200 requests
 Completed 300 requests
@@ -147,8 +147,8 @@ Completed 800 requests
 Completed 900 requests
 
 
-Server Software:        CherryPy/3.0.1alpha
-Server Hostname:        localhost
+Server Software:        CherryPy/3.1beta
+Server Hostname:        127.0.0.1
 Server Port:            8080
 
 Document Path:          /static/index.html
@@ -207,7 +207,9 @@ Finished 1000 requests
         port = cherrypy.server.socket_port
         assert self.concurrency > 0
         assert self.requests > 0
-        return ("-k -n %s -c %s http://localhost:%s%s" %
+        # Don't use "localhost".
+        # Cf http://mail.python.org/pipermail/python-win32/2008-March/007050.html
+        return ("-k -n %s -c %s http://127.0.0.1:%s%s" %
                 (self.requests, self.concurrency, port, self.path))
     
     def run(self):
@@ -306,7 +308,7 @@ def startup_modpython(req=None):
             if ab_opt:
                 global AB_PATH
                 AB_PATH = ab_opt
-        cherrypy.engine.start(blocking=False)
+        cherrypy.engine.start()
     if cherrypy.engine.state == cherrypy._cpengine.STARTING:
         cherrypy.engine.wait()
     return 0 # apache.OK
@@ -363,7 +365,7 @@ if __name__ == '__main__':
         # can be tested from a standard web browser.
         def run():
             port = cherrypy.server.socket_port
-            print ("You may now open http://localhost:%s%s/" %
+            print ("You may now open http://127.0.0.1:%s%s/" %
                    (port, SCRIPT_NAME))
             
             if "--null" in opts:
@@ -377,8 +379,7 @@ if __name__ == '__main__':
             try:
                 run_standard_benchmarks()
             finally:
-                cherrypy.engine.stop()
-                cherrypy.server.stop()
+                cherrypy.engine.exit()
     
     print "Starting CherryPy app server..."
     
@@ -399,6 +400,5 @@ if __name__ == '__main__':
             cherrypy.server.request_class = NullRequest
             cherrypy.server.response_class = NullResponse
         
-        cherrypy.server.quickstart()
-        # This will block
         cherrypy.engine.start_with_callback(run)
+        cherrypy.engine.block()
