@@ -3,15 +3,16 @@
 from cherrypy.test import test
 test.prefer_parent_path()
 
+import os
+localDir = os.path.dirname(__file__)
+import sys
+import types
+
 import cherrypy
 from cherrypy import _cptools, tools
 from cherrypy.lib import http, static
-import types
 
-import os
-localDir = os.path.dirname(__file__)
-log_file = os.path.join(localDir, "test.log")
-log_access_file = os.path.join(localDir, "access.log")
+
 favicon_path = os.path.join(os.getcwd(), localDir, "../favicon.ico")
 
 defined_http_methods = ("OPTIONS", "GET", "HEAD", "POST", "PUT", "DELETE",
@@ -49,6 +50,10 @@ def setup_server():
         def upload(self, file):
             return "Size: %s" % len(file.file.read())
         upload.exposed = True
+        
+        def baseurl(self, path_info, relative=None):
+            return cherrypy.url(path_info, relative=bool(relative))
+        baseurl.exposed = True
     
     root = Root()
     
@@ -58,7 +63,7 @@ def setup_server():
         and adds an instance of the subclass as an attribute of root.
         """
         def __init__(cls, name, bases, dct):
-            type.__init__(name, bases, dct)
+            type.__init__(cls, name, bases, dct)
             for value in dct.itervalues():
                 if isinstance(value, types.FunctionType):
                     value.exposed = True
@@ -72,10 +77,14 @@ def setup_server():
         _cp_config = {'tools.trailing_slash.on': False}
         
         def index(self, path_info, relative=None):
-            return cherrypy.url(path_info, relative=bool(relative))
+            if relative != 'server':
+                relative = bool(relative)
+            return cherrypy.url(path_info, relative=relative)
         
         def leaf(self, path_info, relative=None):
-            return cherrypy.url(path_info, relative=bool(relative))
+            if relative != 'server':
+                relative = bool(relative)
+            return cherrypy.url(path_info, relative=relative)
     
     
     class Params(Test):
@@ -162,6 +171,11 @@ def setup_server():
         def index(self):
             raise cherrypy.InternalRedirect("/")
         
+        def choke(self):
+            return 3 / 0
+        choke.exposed = True
+        choke._cp_config = {'hooks.before_error_response': redir_custom}
+        
         def relative(self, a, b):
             raise cherrypy.InternalRedirect("cousin?t=6")
         
@@ -191,7 +205,6 @@ def setup_server():
         
         def login(self):
             return "Please log in"
-        login._cp_config = {'hooks.before_error_response': redir_custom}
         
         def custom_err(self):
             return "Something went horribly wrong."
@@ -200,6 +213,7 @@ def setup_server():
             return "whatever"
         early_ir._cp_config = {'hooks.before_request_body': redir_custom}
     
+    
     class Image(Test):
         
         def getImagesByUser(self, user_id):
@@ -224,16 +238,26 @@ def setup_server():
         def as_refyield(self):
             for chunk in self.as_yield():
                 yield chunk
-
-
+    
+    
+    def callable_error_page(status, **kwargs):
+        return "Error %s - Well, I'm very sorry but you haven't paid!" % status
+    
+    
     class Error(Test):
         
         _cp_config = {'tools.log_tracebacks.on': True,
                       }
         
-        def custom(self):
-            raise cherrypy.HTTPError(404, "No, <b>really</b>, not found!")
-        custom._cp_config = {'error_page.404': os.path.join(localDir, "static/index.html")}
+        def custom(self, err='404'):
+            raise cherrypy.HTTPError(int(err), "No, <b>really</b>, not found!")
+        custom._cp_config = {'error_page.404': os.path.join(localDir, "static/index.html"),
+                             'error_page.401': callable_error_page,
+                             }
+        
+        def custom_default(self):
+            return 1 + 'a' # raise an unexpected error
+        custom_default._cp_config = {'error_page.default': callable_error_page}
         
         def noexist(self):
             raise cherrypy.HTTPError(404, "No, <b>really</b>, not found!")
@@ -392,14 +416,16 @@ def setup_server():
             cherrypy.request.asdf = "rassfrassin"
             return existing
     
+    if sys.version_info >= (2, 5):
+        from cherrypy.test import py25
+        Root.expose_dec = py25.ExposeExamples()
+    
     cherrypy.config.update({
-        'log.error_file': log_file,
         'environment': 'test_suite',
         'server.max_request_body_size': 200,
         'server.max_request_header_size': 500,
         })
     appconf = {
-        '/': {'log.access_file': log_access_file},
         '/method': {'request.methods_with_bodies': ("POST", "PUT", "PROPFIND")},
         }
     cherrypy.tree.mount(root, config=appconf)
@@ -464,73 +490,6 @@ class CoreRequestHandlingTest(helper.CPWebCase):
         msg = "Illegal response status from server ('error' is non-numeric)."
         self.assertErrorPage(500, msg)
     
-    def testLogging(self):
-        f = open(log_access_file, "wb")
-        f.write("")
-        f.close()
-        f = open(log_file, "wb")
-        f.write("")
-        f.close()
-        
-        self.getPage("/flatten/as_string")
-        self.assertBody('content')
-        self.assertStatus(200)
-        
-        self.getPage("/flatten/as_yield")
-        self.assertBody('content')
-        self.assertStatus(200)
-        
-        data = open(log_access_file, "rb").readlines()
-        
-        host = self.HOST
-        if not host:
-            # The empty string signifies INADDR_ANY,
-            # which should respond on localhost.
-            host = "127.0.0.1"
-        intro = '%s - - [' % host
-        
-        if not data[0].startswith(intro):
-            self.fail("%r doesn't start with %r" % (data[0], intro))
-        haslength = False
-        for k, v in self.headers:
-            if k.lower() == 'content-length':
-                haslength = True
-        line = data[-2].strip()
-        if haslength:
-            if not line.endswith('] "GET %s/flatten/as_string HTTP/1.1" 200 7 "" ""'
-                                 % self.prefix()):
-                self.fail(line)
-        else:
-            if not line.endswith('] "GET %s/flatten/as_string HTTP/1.1" 200 - "" ""'
-                                 % self.prefix()):
-                self.fail(line)
-        
-        if not data[-1].startswith(intro):
-            self.fail("%r doesn't start with %r" % (data[-1], intro))
-        haslength = False
-        for k, v in self.headers:
-            if k.lower() == 'content-length':
-                haslength = True
-        line = data[-1].strip()
-        if haslength:
-            self.assert_(line.endswith('] "GET %s/flatten/as_yield HTTP/1.1" 200 7 "" ""'
-                                          % self.prefix()))
-        else:
-            self.assert_(line.endswith('] "GET %s/flatten/as_yield HTTP/1.1" 200 - "" ""'
-                                          % self.prefix()))
-        
-        ignore = helper.webtest.ignored_exceptions
-        ignore.append(ValueError)
-        try:
-            # Test that tracebacks get written to the error log.
-            self.getPage("/error/page_method")
-            self.assertInBody("raise ValueError()")
-            data = open(log_file, "rb").readlines()
-            self.assertEqual(data[0].strip().endswith('HTTP Traceback (most recent call last):'), True)
-            self.assertEqual(data[-3].strip().endswith('raise ValueError()'), True)
-        finally:
-            ignore.pop()
-    
     def testSlashes(self):
         # Test that requests for index methods without a trailing slash
         # get redirected to the same URI path with a trailing slash.
@@ -660,7 +619,7 @@ class CoreRequestHandlingTest(helper.CPWebCase):
         self.assertStatus(200)
         
         # InternalRedirect on error
-        self.getPage("/internalredirect/login/illegal/extra/vpath/atoms")
+        self.getPage("/internalredirect/choke")
         self.assertStatus(200)
         self.assertBody("Something went horribly wrong.")
     
@@ -686,11 +645,17 @@ class CoreRequestHandlingTest(helper.CPWebCase):
             self.getPage("/error/page_yield")
             self.assertErrorPage(500, pattern=valerr)
             
-            self.getPage("/error/page_streamed")
-            # Because this error is raised after the response body has
-            # started, the status should not change to an error status.
-            self.assertStatus(200)
-            self.assertBody("word upUnrecoverable error in the server.")
+            if cherrypy.server.protocol_version == "HTTP/1.0":
+                self.getPage("/error/page_streamed")
+                # Because this error is raised after the response body has
+                # started, the status should not change to an error status.
+                self.assertStatus(200)
+                self.assertBody("word up")
+            else:
+                # Under HTTP/1.1, the chunked transfer-coding is used.
+                # The HTTP client will choke when the output is incomplete.
+                self.assertRaises(ValueError, self.getPage,
+                                  "/error/page_streamed")
             
             # No traceback should be present
             self.getPage("/error/cause_err_in_finalize")
@@ -699,18 +664,28 @@ class CoreRequestHandlingTest(helper.CPWebCase):
         finally:
             ignore.pop()
         
-        # Test custom error page.
+        # Test custom error page for a specific error.
         self.getPage("/error/custom")
         self.assertStatus(404)
         self.assertBody("Hello, world\r\n" + (" " * 499))
         
+        # Test custom error page for a specific error.
+        self.getPage("/error/custom?err=401")
+        self.assertStatus(401)
+        self.assertBody("Error 401 Unauthorized - Well, I'm very sorry but you haven't paid!")
+        
+        # Test default custom error page.
+        self.getPage("/error/custom_default")
+        self.assertStatus(500)
+        self.assertBody("Error 500 Internal Server Error - Well, I'm very sorry but you haven't paid!".ljust(513))
+        
         # Test error in custom error page (ticket #305).
         # Note that the message is escaped for HTML (ticket #310).
         self.getPage("/error/noexist")
         self.assertStatus(404)
         msg = ("No, &lt;b&gt;really&lt;/b&gt;, not found!<br />"
                "In addition, the custom error page failed:\n<br />"
-               "[Errno 2] No such file or directory: 'nonexistent.html'")
+               "IOError: [Errno 2] No such file or directory: 'nonexistent.html'")
         self.assertInBody(msg)
         
         if (hasattr(self, 'harness') and
@@ -776,7 +751,7 @@ class CoreRequestHandlingTest(helper.CPWebCase):
         self.getPage("/headerelements/get_elements?headername=Expect", [e])
         self.assertBody('100-continue')
         
-        self.getPage("/expect/expectation_failed", [('Content-Length', '200'), e])
+        self.getPage("/expect/expectation_failed", [e])
         self.assertStatus(417)
     
     def testHeaderElements(self):
@@ -826,6 +801,13 @@ class CoreRequestHandlingTest(helper.CPWebCase):
         self.assertBody("da\n"
                         "en-gb;q=0.8\n"
                         "en;q=0.7")
+        
+        # Test malformed header parsing. See http://www.cherrypy.org/ticket/763.
+        self.getPage("/headerelements/get_elements?headername=Content-Type",
+                     # Note the illegal trailing ";"
+                     headers=[('Content-Type', 'text/html; charset=utf-8;')])
+        self.assertStatus(200)
+        self.assertBody("text/html;charset=utf-8")
     
     def testHeaders(self):
         # Tests that each header only appears once, regardless of case.
@@ -834,7 +816,7 @@ class CoreRequestHandlingTest(helper.CPWebCase):
         hnames = [name.title() for name, val in self.headers]
         for key in ['Content-Length', 'Content-Type', 'Date',
                     'Expires', 'Location', 'Server']:
-            self.assertEqual(hnames.count(key), 1)
+            self.assertEqual(hnames.count(key), 1, self.headers)
         
         if cherrypy.server.protocol_version == "HTTP/1.1":
             # Test RFC-2047-encoded request and response header values
@@ -870,7 +852,7 @@ class CoreRequestHandlingTest(helper.CPWebCase):
         self.getPage("/headers/Content-Type",
                      headers=[("Content-type", "application/json")])
         self.assertBody("application/json")
-        
+    
     def testHTTPMethods(self):
         helper.webtest.methods_with_bodies = ("POST", "PUT", "PROPFIND")
         
@@ -900,12 +882,32 @@ class CoreRequestHandlingTest(helper.CPWebCase):
         self.assertStatus(200)
         self.assertBody(b)
         
+        # Request a PUT method with a file body but no Content-Type.
+        # See http://www.cherrypy.org/ticket/790.
+        b = "one thing on top of another"
+        self.persistent = True
+        try:
+            conn = self.HTTP_CONN
+##            conn.set_debuglevel(10)
+            conn.putrequest("PUT", "/method/request_body", skip_host=True)
+            conn.putheader("Host", self.HOST)
+            conn.putheader('Content-Length', str(len(b)))
+            conn.endheaders()
+            conn.send(b)
+            response = conn.response_class(conn.sock, method="PUT")
+            response.begin()
+            self.assertEqual(response.status, 200)
+            self.body = response.read()
+            self.assertBody(b)
+        finally:
+            self.persistent = False
+        
         # Request a PUT method with no body whatsoever (not an empty one).
         # See http://www.cherrypy.org/ticket/650.
         # Provide a C-T or webtest will provide one (and a C-L) for us.
         h = [("Content-Type", "text/plain")]
         self.getPage("/method/reachable", headers=h, method="PUT")
-        self.assertBody("success")
+        self.assertStatus(411)
         
         # Request a custom method with a request body
         b = ('<?xml version="1.0" encoding="utf-8" ?>\n\n'
@@ -947,7 +949,6 @@ class CoreRequestHandlingTest(helper.CPWebCase):
         self.assertBody(data)
     
     def testCookies(self):
-        import sys
         if sys.version_info >= (2, 5):
             self.getPage("/cookies/single?name=First",
                          [('Cookie', 'First=Dinsdale;')])
@@ -970,8 +971,9 @@ class CoreRequestHandlingTest(helper.CPWebCase):
             self.assertHeader('Set-Cookie', 'Last=Piranha;')
     
     def testMaxRequestSize(self):
-        self.getPage("/", headers=[('From', "x" * 500)])
-        self.assertStatus(413)
+        for size in (500, 5000, 50000):
+            self.getPage("/", headers=[('From', "x" * 500)])
+            self.assertStatus(413)
         
         # Test for http://www.cherrypy.org/ticket/421
         # (Incorrect border condition in readline of SizeCheckWrapper).
@@ -1054,6 +1056,66 @@ Content-Type: text/plain
         self.assertBody('../page1')
         self.getPage('/url/?path_info=other/../page1&relative=True')
         self.assertBody('page1')
+        
+        # Output relative to /
+        self.getPage('/baseurl?path_info=ab&relative=True')
+        self.assertBody('ab')
+        # Output relative to /
+        self.getPage('/baseurl?path_info=/ab&relative=True')
+        self.assertBody('ab')
+        
+        # absolute-path references ("server-relative")
+        # Input relative to current
+        self.getPage('/url/leaf?path_info=page1&relative=server')
+        self.assertBody('/url/page1')
+        self.getPage('/url/?path_info=page1&relative=server')
+        self.assertBody('/url/page1')
+        # Input is 'absolute'; that is, relative to script_name
+        self.getPage('/url/leaf?path_info=/page1&relative=server')
+        self.assertBody('/page1')
+        self.getPage('/url/?path_info=/page1&relative=server')
+        self.assertBody('/page1')
+    
+    def test_expose_decorator(self):
+        if not sys.version_info >= (2, 5):
+            print "skipped (Python 2.5+ only)",
+            return
+        
+        # Test @expose
+        self.getPage("/expose_dec/no_call")
+        self.assertStatus(200)
+        self.assertBody("Mr E. R. Bradshaw")
+        
+        # Test @expose()
+        self.getPage("/expose_dec/call_empty")
+        self.assertStatus(200)
+        self.assertBody("Mrs. B.J. Smegma")
+        
+        # Test @expose("alias")
+        self.getPage("/expose_dec/call_alias")
+        self.assertStatus(200)
+        self.assertBody("Mr Nesbitt")
+        # Does the original name work?
+        self.getPage("/expose_dec/nesbitt")
+        self.assertStatus(200)
+        self.assertBody("Mr Nesbitt")
+        
+        # Test @expose(["alias1", "alias2"])
+        self.getPage("/expose_dec/alias1")
+        self.assertStatus(200)
+        self.assertBody("Mr Ken Andrews")
+        self.getPage("/expose_dec/alias2")
+        self.assertStatus(200)
+        self.assertBody("Mr Ken Andrews")
+        # Does the original name work?
+        self.getPage("/expose_dec/andrews")
+        self.assertStatus(200)
+        self.assertBody("Mr Ken Andrews")
+        
+        # Test @expose(alias="alias")
+        self.getPage("/expose_dec/alias3")
+        self.assertStatus(200)
+        self.assertBody("Mr. and Mrs. Watson")
 
 
 if __name__ == '__main__':
