@@ -46,6 +46,7 @@ def setup_server():
                         'tools.basic_auth.realm': 'localhost',
                         'tools.basic_auth.users': fetch_password,
                         'tools.basic_auth.encrypt': sha_password_encrypter}}
+            
     root = Root()
     root.digest = DigestProtected()
     root.basic = BasicProtected()
@@ -65,11 +66,11 @@ class HTTPAuthTest(helper.CPWebCase):
 
     def testBasic(self):
         self.getPage("/basic/")
-        self.assertStatus('401 Unauthorized')
+        self.assertStatus(401)
         self.assertHeader('WWW-Authenticate', 'Basic realm="localhost"')
 
         self.getPage('/basic/', [('Authorization', 'Basic dGVzdDp0ZX60')])
-        self.assertStatus('401 Unauthorized')
+        self.assertStatus(401)
         
         self.getPage('/basic/', [('Authorization', 'Basic dGVzdDp0ZXN0')])
         self.assertStatus('200 OK')
@@ -77,11 +78,11 @@ class HTTPAuthTest(helper.CPWebCase):
 
     def testBasic2(self):
         self.getPage("/basic2/")
-        self.assertStatus('401 Unauthorized')
+        self.assertStatus(401)
         self.assertHeader('WWW-Authenticate', 'Basic realm="localhost"')
 
         self.getPage('/basic2/', [('Authorization', 'Basic dGVzdDp0ZX60')])
-        self.assertStatus('401 Unauthorized')
+        self.assertStatus(401)
         
         self.getPage('/basic2/', [('Authorization', 'Basic dGVzdDp0ZXN0')])
         self.assertStatus('200 OK')
@@ -89,7 +90,7 @@ class HTTPAuthTest(helper.CPWebCase):
 
     def testDigest(self):
         self.getPage("/digest/")
-        self.assertStatus('401 Unauthorized')
+        self.assertStatus(401)
         
         value = None
         for k, v in self.headers:
@@ -128,11 +129,21 @@ class HTTPAuthTest(helper.CPWebCase):
         elif tokens['qop'] != '"auth"':
             self._handlewebError(bad_value_msg % ('qop', '"auth"', tokens['qop']))
 
-            # now let's see if what 
+        # Test a wrong 'realm' value
+        base_auth = 'Digest username="test", realm="wrong realm", nonce="%s", uri="/digest/", algorithm=MD5, response="%s", qop=auth, nc=%s, cnonce="1522e61005789929"'
+
+        auth = base_auth % (nonce, '', '00000001')
+        params = httpauth.parseAuthorization(auth)
+        response = httpauth._computeDigestResponse(params, 'test')
+        
+        auth = base_auth % (nonce, response, '00000001')
+        self.getPage('/digest/', [('Authorization', auth)])
+        self.assertStatus('401 Unauthorized')
+
+        # Test that must pass
         base_auth = 'Digest username="test", realm="localhost", nonce="%s", uri="/digest/", algorithm=MD5, response="%s", qop=auth, nc=%s, cnonce="1522e61005789929"'
 
         auth = base_auth % (nonce, '', '00000001')
-                
         params = httpauth.parseAuthorization(auth)
         response = httpauth._computeDigestResponse(params, 'test')
         
